'use strict';

const crypto = require('node:crypto');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const crypto__default = /*#__PURE__*/_interopDefaultCompat(crypto);

const uniqueId = typeof window !== "undefined" && window.crypto ? window.crypto.randomUUID() : crypto__default.randomUUID();
const variablePlaceholder = new RegExp('(\\\\)?"@__(F|R|D|M|S|A|U|I|B|L|E)-' + uniqueId + '-(\\d+)__@"', "g");
const isNativeCode = /\{\s*\[native code\]\s*\}/g;
const isNativeFunction = /function.*?\(/;
const isArrowFunction = /.*?=>.*?/;
const unsafeCharacters = /[<>\/\u2028\u2029]/g;
const reserved = ["*", "async"];
const escapedCharacters = {
  "<": "\\u003C",
  ">": "\\u003E",
  "/": "\\u002F",
  "\u2028": "\\u2028",
  "\u2029": "\\u2029"
};
function escapeUnsafeChars(unsafeChar) {
  return escapedCharacters[unsafeChar];
}
function removeFunctions(object) {
  for (const key in object) {
    if (typeof object[key] === "function") {
      delete object[key];
    }
  }
}
function serializeFunction(fn) {
  const serializedFn = fn.toString();
  if (isNativeCode.test(serializedFn)) {
    throw new TypeError("Serializing native function: " + fn.name);
  }
  if (isNativeFunction.test(serializedFn)) {
    return serializedFn;
  }
  if (isArrowFunction.test(serializedFn)) {
    return serializedFn;
  }
  const argsStartsAt = serializedFn.indexOf("(");
  const def = serializedFn.slice(0, argsStartsAt).trim().split(" ").filter((val) => val.length > 0);
  const nonReservedSymbols = def.filter((val) => !reserved.includes(val));
  if (nonReservedSymbols.length > 0) {
    const asyncIndex = def.indexOf("async");
    return `${asyncIndex > -1 ? "async " : ""}function${def.join("").indexOf("*") > -1 ? "*" : ""}${serializedFn.slice(argsStartsAt)}`;
  } else {
    return serializedFn;
  }
}
function serialize(object, options = {}) {
  if (typeof options === "number" || typeof options === "string") {
    options = { space: options };
  }
  const collections = {
    functions: [],
    regexps: [],
    dates: [],
    maps: [],
    sets: [],
    arrays: [],
    undefs: [],
    infinities: [],
    bigInts: [],
    urls: [],
    errors: []
  };
  function replacer(key, value) {
    if (options.ignoreFunctions) {
      removeFunctions(value);
    }
    if (!value && value !== void 0 && value !== BigInt(0)) {
      return value;
    }
    const originalValue = this[key];
    const type = typeof originalValue;
    let placeholderType = "";
    let name = void 0;
    if (type === "object" && originalValue !== null) {
      if (originalValue instanceof RegExp) {
        placeholderType = "R";
        name = "regexps";
      } else if (originalValue instanceof Date) {
        placeholderType = "D";
        name = "dates";
      } else if (originalValue instanceof Map) {
        placeholderType = "M";
        name = "maps";
      } else if (originalValue instanceof Set) {
        placeholderType = "S";
        name = "sets";
      } else if (originalValue instanceof Array) {
        const isSparse = originalValue.filter(() => true).length !== originalValue.length;
        if (isSparse) {
          placeholderType = "A";
          name = "arrays";
        }
      } else if (originalValue instanceof URL) {
        placeholderType = "L";
        name = "urls";
      } else if (originalValue instanceof Error) {
        placeholderType = "E";
        name = "errors";
      }
    } else if (type === "function") {
      placeholderType = "F";
      name = "functions";
    } else if (type === "undefined") {
      placeholderType = "U";
      name = "undefs";
    } else if (type === "number" && !isFinite(originalValue)) {
      placeholderType = "I";
      name = "infinities";
    } else if (type === "bigint") {
      placeholderType = "B";
      name = "bigInts";
    }
    if (placeholderType !== "") {
      return `@__${placeholderType}-${uniqueId}-${collections[name].push(originalValue) - 1}__@`;
    } else {
      return value;
    }
  }
  if (options.ignoreFunctions && typeof object === "function") {
    object = void 0;
  }
  if (object === void 0) {
    return String(object);
  }
  let str;
  if (options.plain && !options.space) {
    str = JSON.stringify(object);
  } else {
    str = JSON.stringify(object, options.plain ? void 0 : replacer, options.space);
  }
  if (typeof str !== "string") {
    return String(str);
  }
  if (options.unsafe !== true) {
    str = str.replace(unsafeCharacters, escapeUnsafeChars);
  }
  if (Object.values(collections).every((arr) => arr.length === 0)) {
    return str;
  }
  const serializedFunctions = collections.functions.map(serializeFunction);
  return str.replace(variablePlaceholder, (match, backSlash, type, valueIndex) => {
    if (backSlash) {
      return match;
    }
    switch (type) {
      case "D":
        return `new Date("${collections.dates[valueIndex].toISOString()}")`;
      case "R": {
        const regex = collections.regexps[valueIndex];
        return `new RegExp(${serialize(regex.source)}, "${regex.flags}")`;
      }
      case "M":
        return `new Map(${serialize(Array.from(collections.maps[valueIndex].entries()), options)})`;
      case "S":
        return `new Set(${serialize(Array.from(collections.sets[valueIndex].values()), options)})`;
      case "A": {
        const array = collections.arrays[valueIndex];
        return `Array.prototype.slice.call(${serialize(Object.assign({ length: array.length }, array), options)})`;
      }
      case "U":
        return "undefined";
      case "I":
        return collections.infinities[valueIndex];
      case "B":
        return `BigInt("${collections.bigInts[valueIndex]}")`;
      case "L":
        return `new URL("${collections.urls[valueIndex].toString()}")`;
      case "F":
        return serializedFunctions[valueIndex];
      case "E":
        return `new Error("${collections.errors[valueIndex].message}")`;
      default:
        return match;
    }
  });
}

exports.serialize = serialize;
